package ueb071bildmanipulation;

import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.filechooser.FileNameExtensionFilter;

import processing.core.PApplet;
import processing.core.PImage;

@SuppressWarnings("serial")
public class Ueb071BildManipulation extends PApplet {

	private PImage sourceImage;
	private PImage showImage;
	
	public void setup() {
		
		//sourceImage = loadFile();
		sourceImage = loadImage("kaefer.png");
		sourceImage.loadPixels();
		
		if (sourceImage == null) {
			exit();
		}
		
		showImage = sourceImage;
		noLoop();
	}	

	public void draw() {
		
		size(showImage.width, showImage.height);
		
		image(showImage, width / 2 - showImage.width / 2, height / 2 - showImage.height / 2);
		
		fill(color(255, 255, 255, 150));
		
		// Show text	
		textSize(20);
		text("g\ni\ns\nl", 20, 40);
		text("Graustufen\nInvertieren\nQuellbild\nBild laden", 35, 40);
	}
	
	public void keyPressed() {
		
		switch(key) {
		case 's':
			
			// Zeige Quellbild (Source)
			showImage = sourceImage;
			break;
		case 'i':
			
			// Invertiere aktuelles Bild und zeige dieses an
			showImage = invert(showImage);
			break;
		case 'g':
			
			// Wandle das aktuelle Bild in ein Graubild um zeige dieses an
			showImage = gray(showImage);
			break;
		case 'l':
			
			// Bild laden
			PImage newImage = loadFile();
			
			if (newImage == null) return;
			
			sourceImage = newImage;
			showImage = sourceImage;
			break;
		case 'd':
			
			// Dunkle das Bild ab
			showImage = changeBrightness(showImage, -5);
			break;
		case 'h':
			
			// Helle das Bild auf
			showImage = changeBrightness(showImage, 5);
			break;
		}		
		

		redraw();
	}
	
	
	private PImage loadFile() {
		
		//Create a file chooser
		final JFileChooser fileChooser = new JFileChooser();
		
		FileNameExtensionFilter filter = new FileNameExtensionFilter("JPEG Dateien", "jpg", "jpeg");
		fileChooser.setFileFilter(filter);
		
		int returnVal = fileChooser.showOpenDialog(this);
		
		if (returnVal == JFileChooser.APPROVE_OPTION) {
            File file = fileChooser.getSelectedFile();
            
            return loadImage(file.getPath());
        } else {
            System.out.println("Open command cancelled by user.");
        }
		
		return null;
	}
	
	/**
	 * Invertiert die Farben eines Bildes
	 * @param source
	 * @return Bild mit invertierten Farben
	 */
	private PImage invert(PImage source) {
		
		PImage image = new PImage(source.width, source.height);
		
		for (int posX = 0; posX < source.width; posX++) {
			for (int posY = 0; posY < source.height; posY++) {
							
				int color = source.pixels[posX + posY * width];
				
				// Farben auslesen (ARGB)
				int a = (color >> 24) & 0xFF;
				int r = (color >> 16) & 0xFF;
				int g = (color >> 8) & 0xFF;
				int b = color & 0xFF;
				
				r = 255 - r;
				g = 255 - g;
				b = 255 - b;

				// Farbe setzen
				image.set(posX, posY, (a << 24) + (r << 16) + (g << 8) + b);				
			}
		}
		
		return image;
	}
	
	/**
	 * Wandelt ein Bild in ein Graubild um
	 * @param source
	 * @return Ein Graubild
	 */
	private PImage gray(PImage source) {
		
		PImage image = new PImage(source.width, source.height);
		image.loadPixels();
				
		for (int posX = 0; posX < source.width; posX++) {
			for (int posY = 0; posY < source.height; posY++) {
							
				int color = source.pixels[posX + posY * width];
				
				// Farben auslesen (ARGB)
				int a = (color >> 24) & 0xFF;
				int r = (color >> 16) & 0xFF;
				int g = (color >> 8) & 0xFF;
				int b = color & 0xFF;
				
				int average = (int)((r + g + b) / 3f);

				// Farbe setzen
				image.set(posX, posY, (a << 24) + (average << 16) + (average << 8) + average);				
			}
		}
		
		return image;
	}
	
	/**
	 * ndert die Helligkeit eines Bildes
	 * @param source
	 * @return Gendertes Bild
	 */
	private PImage changeBrightness(PImage source, int value) {
		
		PImage image = new PImage(source.width, source.height);
		image.loadPixels();
				
		for (int posX = 0; posX < source.width; posX++) {
			for (int posY = 0; posY < source.height; posY++) {
							
				int color = source.pixels[posX + posY * width];
				
				// Farben auslesen (ARGB)
				int a = (color >> 24) & 0xFF;
				
				int[] parts = new int[3];
				for (int index = 0; index < 3; index++) {
					parts[index] = (color >> index * 8) & 0xFF;
					
					parts[index] += value;
					
					if (parts[index] > 255) parts [index] = 255;
					else if (parts[index] < 0) parts[index] = 0;
				}								

				// Farbe setzen
				image.set(posX, posY, (a << 24) + (parts[2] << 16) + (parts[1] << 8) + parts[0]);				
			}
		}
		
		return image;
	}
	
	
}
